module crcCheck (clk, dataIn, dataOut, incoming, busy, EXTERNAL_RESET, badCheck,crcNibbles);

input clk;
input [3:0] dataIn;
input incoming;
input EXTERNAL_RESET;
output [3:0] dataOut;
output busy;
output badCheck;
output crcNibbles;

reg [2:0] crcCounter;   // Used to put the last bits and crc on the end
reg [31:0] crc;		// Register for holding the CRC calculation
reg reset;		// Used for determining when to reset
reg state;		// Tracking when we're are outputting stuff
wire badCheck;		// Signifies that the CRC check on incoming data was successful
reg CRCCheck;		// Set when the CRC was OK
wire [3:0] dataOut;	// Output (could be straight data, could be crc)
wire busy;		// Signals that we are outputting relevant data
wire [3:0] in;		// For special CRC things
reg crcNibbles;		// This is asserted if what is being outputted is the CRC calculation


// These are special CRC things, don't ask
assign in[3] = (dataIn[3] ^ crc[28]); 
assign in[2] = (dataIn[2] ^ crc[29]);
assign in[1] = (dataIn[1] ^ crc[30]);
assign in[0] = (dataIn[0] ^ crc[31]);

// Stream through data or send out CRC
assign dataOut = (incoming) ? dataIn : ({crc[28],crc[29],crc[30],crc[31]} ^ 'hf);

// Signal that valid data is popping out
assign busy = (incoming || (state && !reset));

// Signal during CRC write if it was a good CRC
assign badCheck = ((crc != 'hc704dd7b) && !incoming && state && (crcCounter == 0) && !reset);

always @(posedge clk) begin
// Always shovelling in data and shifting when good data is available
// Always computing CRC when good data is coming in    

  crcNibbles <= (busy & ~incoming);

  if(EXTERNAL_RESET) begin
    state <= 0;
    crcCounter <= 0;
    crc <= 'hffffffff;
    CRCCheck <= 0;

  end
  
  // CRC stuff
  if(incoming == 1) begin
    crc[0] <= in[3];
    crc[1] <= in[3] ^ in[2];
    crc[2] <= in[3] ^ in[2] ^ in[1];
    crc[3] <= in[2] ^ in[1] ^ in[0];
    crc[4] <= in[3] ^ in[1] ^ in[0] ^ crc[0];
    crc[5] <= in[3] ^ in[2] ^ in[0] ^ crc[1];
    crc[6] <= in[2] ^ in[1] ^ crc[2];
    crc[7] <= in[3] ^ in[1] ^ in[0] ^ crc[3];
    crc[8] <= in[3] ^ in[2] ^ in[0] ^ crc[4];
    crc[9] <= in[2] ^ in[1] ^ crc[5];
    crc[10] <= in[3] ^ in[1] ^ in[0] ^ crc[6];
    crc[11] <= in[3] ^ in[2] ^ in[0] ^ crc[7];
    crc[12] <= in[3] ^ in[2] ^ in[1] ^ crc[8];
    crc[13] <= in[2] ^ in[1] ^ in[0] ^ crc[9];
    crc[14] <= in[1] ^ in[0] ^ crc[10];
    crc[15] <= in[0] ^ crc[11];
    crc[16] <= in[3] ^ crc[12];
    crc[17] <= in[2] ^ crc[13];
    crc[18] <= in[1] ^ crc[14];
    crc[19] <= in[0] ^ crc[15];
    crc[20] <= crc[16];
    crc[21] <= crc[17];
    crc[22] <= in[3] ^ crc[18];
    crc[23] <= in[3] ^ in[2] ^ crc[19];
    crc[24] <= in[2] ^ in[1] ^ crc[20];
    crc[25] <= in[1] ^ in[0] ^ crc[21];
    crc[26] <= in[3] ^ in[0] ^ crc[22];
    crc[27] <= in[2] ^ crc[23];
    crc[28] <= in[1] ^ crc[24];
    crc[29] <= in[0] ^ crc[25];
    crc[30] <= crc[26];
    crc[31] <= crc[27];
  end

// Control buffering and sending CRC

  // State 0: Waiting for new data to come in
  if(state == 0) begin
    if(incoming == 1) 
      state <= 1;			// Move to state 1 when data starts coming in
    else begin
      crc <= 'hffffffff;		// Reset CRC
    end
  end
  // State 1:  We are performing CRC on incoming data or writing out the CRC
  else begin
    // Here data is coming in so we are just writing out the data and doing CRC
    if(incoming == 0) begin
      if(reset == 1) begin // Time to reset
      	state <= 0;
        reset <= 0;
        crcCounter <= 0;
        CRCCheck <= 0;
      end
      if(crcCounter == 7) begin		// Reset when the CRC has been written
          reset <= 1;
      end else
      if(crcCounter == 0)
        CRCCheck <= (crc == 'hc704dd7b);
      crcCounter <= crcCounter + 1;	// Increment counter, tracking CRC output
      crc <= crc << 4;			// Shift for next part of CRC
    end
  end

end

endmodule 



