module DataReceiver (RXCLK, RXDV, DATAIN, VALID_OUT, TYPE, OURMAC, BDCAST, LENGTH, BUSY, DEST, SRC, DATA_CRC);

input RXCLK, RXDV; 	// Clock and Received Data
input [3:0] DATAIN; 	// Data incoming
output [3:0] VALID_OUT; // 4-bit chunks of good data
output TYPE;		// Output is part of Type Field as found in packet
output OURMAC;		// Was this for us?
output BDCAST;		// Was this a broadcast packet?
output [15:0]LENGTH;	// How long is this packet (bytes) -- Actual data + CRC
output BUSY;		// We are processing a packet
output DEST;		// This is the Destination Field
output SRC;		// Here is the SRC Field
output DATA_CRC;	// This is either the data or the CRC (we don't know until done)

reg [47:0] DESTINATION;
reg BUSY;
reg [3:0] VALID_OUT;
reg [2:0] STATE;
reg [3:0] i;
reg [15:0] LENGTH;

reg DEST;
reg SRC;
reg TYPE;
reg DATA_CRC;

// Check our MAC Address
		    ///MAC HERE///|-|-|-|-|-|-|//////
assign OURMAC = ((DESTINATION[47:24] != 'h001122 || DESTINATION[23:0] != 'h334455) & (STATE == 2)) ? 1 : 0;

		   // Destination of all 1's is a broadcast in Ethernet
assign BDCAST = ((DESTINATION[47:24] != 'hFFFFFF || DESTINATION[23:0] != 'hFFFFFF) & (STATE == 2)) ? 1 : 0;

// What are we up to?
always @(posedge RXCLK) begin
	if(STATE == 1) begin      DEST <= RXDV; SRC <= 0; TYPE <= 0; DATA_CRC <= 0; end
	else if(STATE == 2) begin SRC <= 1; DEST <= 0; TYPE <= 0; DATA_CRC <= 0; end
	else if(STATE == 3) begin TYPE <= 1; SRC <= 0; DEST <= 0; DATA_CRC <= 0; end
	else if((STATE == 4 | STATE == 5) & RXDV == 1) begin DATA_CRC <= 1; TYPE <=0; SRC <= 0; DEST <= 0; end
	else begin TYPE <=0; SRC <= 0; DEST <= 0; DATA_CRC <= 0; end
end


// State Machine
always @(posedge RXCLK) begin

	// Pipeline, route the data
	VALID_OUT <= DATAIN;

	// RESET State when RXDV goes low
	if(RXDV == 0) begin
		BUSY <= 0;
		STATE <= 1;
	end
	
	else begin
	
	  // We are busy processing packets
	  BUSY <= 1;
	  
	  case(STATE)
	
	  // Waiting for the SFD "5d"
	  //0: begin
	//	if(DATAIN == 'h5) STATE <= 6;
	  //   end   
	  //6: begin
	 // 	if(DATAIN == 'hd) begin
	 //		STATE <= 1;
	 // 		i <= 0;
	 // 	end
	 // 	else STATE <= 0;
	    // end
	   
	  // Extracting the "Destination" MAC
	  1: begin
	  	if (i < 11) begin
	  		DESTINATION <= (DESTINATION << 4) | DATAIN;
	  		i <= i + 1;
	  	end
	  	else begin
	  		DESTINATION <= (DESTINATION << 4) | DATAIN;
	  		STATE <= 2;
	  		i <= 0;
	  	end
	     end
	     
	   // Extracting the "Source" MAC
	  2: begin
	  	if( i < 11) i <= i + 1;
	  	else begin
	  	 i <= 0;
	  	 STATE <= 3;
	  	end
	     end
	   
	   // Extracting the "Type" field
	  3: begin
	  	if(i < 3) i <= i + 1;
	  	else begin
	  	 STATE <= 4;
	  	 LENGTH <= 0;
	  	end
	     end
	     
	   // Here Comes the Client Data + CRC
	   4: begin
	   	STATE <= 5;
	      end
	     
	   // Still Client Data, Keep track of # of bytes in Len
	   5: begin
	   	STATE <= 4;
	   	LENGTH <= LENGTH + 1;
	      end
	      
	    endcase
	    
	  end
end	   	
   	  			
		
endmodule 
