module IO_Control (clk, reset, rxdv, txen, sending, receiving, dataIn, dataOut);

// Inputs and Outputs
input clk, rxdv, txen, reset;
input [3:0] dataIn;
output sending, receiving;
output [3:0] dataOut;

// add your declarations here
reg [1:0] recvState;		// Receiving state
reg [1:0] sendState;		// Sending state
reg [63:0] sendBuffer;		// Sending buffer
reg [3:0] sendCount;		// Sending preamble counter
reg [3:0] dataOut;		// Output
reg sending, receiving;		// Signals

// Here we go
always @(posedge clk) begin
  if(reset) begin
    recvState <= 0;
    sendState <= 0;
    sendBuffer <= 0;
    sendCount <= 0;
    receiving <= 0;
    sending <= 0;
  end else begin
  
  // Handle receiving (strip off front)
  if(rxdv) begin
    case(recvState)
    	// Getting Preamble
        0:  begin
              if(dataIn == 'h5)
        	recvState <= 1;
            end
        // Got SFD
        1:  begin
              if(dataIn == 'hd)
        	recvState <= 2;
              else
       		recvState <= 0;
       	    end
       	// Getting data
       	2:  begin
       	      dataOut <= dataIn;
       	      receiving <= 1;
       	    end
    endcase
  end else
  // Reset stuff 
  begin
    recvState <= 0;
    receiving <= 0;
  end

  // Handle sending (add to the front)
  if(txen || (sendState == 2)) begin
  
    // Shift into the buffer
    sendBuffer[3:0] <= dataIn;
    sendBuffer[63:4] <= sendBuffer[59:0];
    
    case(sendState)
    	// Sending Preamble
    	0:  begin
    	      if(sendCount == 15)
    	        sendState <= 1;
    	      sendCount <= sendCount + 1;
    	      sending <= 1;
    	      dataOut <= 'h5;
    	    end
    	// Sending SFD
    	1:  begin
    	      sendState <= 2;
    	      dataOut <= 'hd;
    	    end
    	// Sending data
    	2:  begin
    	      dataOut <= sendBuffer[63:4];
    	      // Sending the last block of buffered data
    	      if(!txen) begin
    	        if(sendCount == 15)
    	          sendState <= 0;
    	        sendCount <= sendCount + 1;
    	      end
    	    end
    endcase
  end else
  // Reset stuff
  begin
    sendState <= 0;
    sending <= 0;
    sendCount <= 0;
  end
end 
end
endmodule 
