module SimpleMem (ENABLE, CLK, REC_DATA, DATA_OUT, WE, ADDR, PUT_DATA, RESET, NO, newPac) ;

input CLK, ENABLE;
input [3:0] REC_DATA;
input RESET;
input NO;
output WE;
output [15:0] DATA_OUT;
output [18:0] ADDR;
output PUT_DATA;
//output bad;	// Bad Packet Received
output newPac;	// New, valid packet received

reg WE;
reg [15:0] DATA_OUT;
reg [11:0] WORKING;
reg [1:0] COUNT;
reg WRITELEN;
reg [18:0] ADDR;
reg PUT_DATA;
reg [15:0] LENGTH;	// Length in Bytes
//reg tempBad;
reg bad;
reg newPac;

always @(posedge CLK) begin

	if(NO) 
	  bad <= 1;

	// 'RESET' STATE
	if((~ENABLE & COUNT == 0 & WRITELEN == 0) | RESET) begin
		COUNT <= 0;
		WORKING <=0;
		WE <= 0;
		ADDR <= 0;
		PUT_DATA <= 0;
		WRITELEN <= 0;
		bad <= 0;
		LENGTH <= 0;
		newPac <= 0;
	end
		
		
	// Reorders Half-words from data received in this order:
	// Most Significant Byte, Least significant 4 bits
	// Most Significant Byte, Most Significant 4 bits
	// Least Significant Byte, Least Significant 4 bits
	// Least Significant Byte, Most significant 4 bits
	else if(WRITELEN == 1 & ~ENABLE & COUNT == 1) begin
		COUNT <= 0;
		WRITELEN <= 0;
		WE <= 1;
	        PUT_DATA <= 1;
	       	ADDR <= 0;
	       	DATA_OUT <= LENGTH;
	 	if(~bad)
	 	  newPac <= 1;
	end
	
	else begin
	
	 case(COUNT)
	
	 0: begin
	 	// Set VPR to HIGH on start of a receive, will be set if no is raised
	 	//if(WRITELEN == 0 & ENABLE) 
	 	  //tempBad <= 0;
		WORKING[7:4] <= REC_DATA;
		WRITELEN <= 1;
		WE <= 0;
		PUT_DATA <= 0;
		COUNT <= 1;
	    end
	   
	 1: begin
	 	WORKING[11:8] <= REC_DATA;
	 	COUNT <= 2;
	 	LENGTH <= LENGTH + 1;
	    end
	   
	 2: begin 
		// It is possible to end on any 1 byte.
		// Here, we would just write out what we have so far
	 	if(~ENABLE) begin
	 		DATA_OUT <= {WORKING[11:4], 'h00};
	 		WE <= 1;
	 		PUT_DATA <= 1;
	 		COUNT <= 0;
	 	end
	 	
	 	else begin
		  	WORKING[3:0] <= REC_DATA;
		  	COUNT <= 3;
		end
	    end
	   
	 3: begin
		DATA_OUT<= {WORKING[11:4], REC_DATA, WORKING[3:0]};
		PUT_DATA <= 1;
		WE <= 1;
		COUNT <= 0;
		ADDR <= ADDR + 1;
		LENGTH <= LENGTH + 1;
	    end 
	   
	 endcase

	end
end
	
		
		


endmodule 
